%%% Implementation of a prefix tree in scheme to store music
%%% Taken from http://lilypondblog.org/2014/07/trees-music-and-lilypond/

\version "2.18.2"
%%% \include "typesetting_functions.ly"

%%% the main music tree
musicTree = #'()

%%% function to store music in `musicTree`
putMusic =
#(define-void-function (parser location path content)(string? ly:music?)
   (let ((pl (string->path path)))
     (set! musicTree (tree-set! musicTree pl content))
     ))

%%% function to get music from `musicTree`
getMusic =
#(define-music-function (parser location path)(string?)
   (let* ((pl (string->path path))
          (content (tree-get musicTree pl)))
     (if (ly:music? content) content (make-music 'SequentialMusic 'void #t))
     ))

%%% set a value in a tree -- based on nested association lists
#(define-public (tree-set! tree path val)
   (if (> (length path) 1)
       (let ((tab (assoc (car path) tree)))
         (if (not (pair? tab))(set! tab '((car path))))
         (assoc-set! tree (car path) (tree-set! (cdr tab) (cdr path) val))
         )
       (assoc-set! tree (car path) val))
   )

%%% get a value from a tree -- based on nested association lists
#(define-public (tree-get tree path)
   (let ((ret (assoc (car path) tree)))
     (if (pair? ret)
         (if (> (length path) 1)
             (tree-get (cdr ret) (cdr path))
             (cdr ret))
         #f))
   )

%%% split a string by '/'
#(define-public (string->path str)(string-split str #\/))

%%% helper function to append strings
stringAppend =
#(define-scheme-function (parser location str1 str2)(string? string?)
   (string-append str1 str2))

%%% create one vocal staff
vocalStaff =
#(define-music-function (parser location my-path-to-vocal name-clef)(string? ly:music?)
   ; create a unique name for the voice
   (let ((voice-name my-path-to-vocal))
     #{
       <<
         \new Staff
         \with {\consists "Ambitus_engraver"}
         \new Voice = #voice-name
         <<
         \global
         \incipit {\incBoiler {\getMusic \stringAppend #my-path-to-vocal "/incipit"} {} {}}
         { $name-clef \musBoiler {\getMusic \stringAppend #my-path-to-vocal "/melody"} {} {}}
         >>
         \new Lyrics \lyricsto #voice-name { \getMusic \stringAppend #my-path-to-vocal "/lyrics" }
       >>
     #}))

vocalStaffNoInc =
#(define-music-function (parser location my-path-to-vocal name-clef)(string? ly:music?)
   ; create a unique name for the voice
   (let ((voice-name my-path-to-vocal))
     #{
       <<
         \new Staff
         \with {\consists "Ambitus_engraver"}
         \new Voice = #voice-name
         <<
         \global
         { $name-clef \musBoiler {\getMusic \stringAppend #my-path-to-vocal "/melody"} {} {}}
         >>
         \new Lyrics \lyricsto #voice-name { \getMusic \stringAppend #my-path-to-vocal "/lyrics" }
       >>
     #}))

%%% and now combine four vocal staves to a choir group:
%%% create vocal staffs for Soprano, Alto, Tenore and Basso
SATBGroup =
#(define-music-function (parser location my-path-to-choir)(string?)
   #{
     \shiftDurations #1 #0 {
     \new StaffGroup = ChoirStaff <<
       { \vocalStaff \stringAppend #my-path-to-choir "/superius" {}}
       { \vocalStaff \stringAppend #my-path-to-choir "/altus" {}}
       { \vocalStaff \stringAppend #my-path-to-choir "/tenor" {}}
       { \vocalStaff \stringAppend #my-path-to-choir "/bassus" {}}
     >>
     }
   #})

Trio =
#(define-music-function (parser location my-path-to-choir v-one v-two v-three)(string? string? string? string?)
   #{
     \shiftDurations #1 #0 {
     \new StaffGroup = ChoirStaff <<
       { \vocalStaff \stringAppend #my-path-to-choir #v-one {}}
       { \vocalStaff \stringAppend #my-path-to-choir #v-two {}}
       { \vocalStaff \stringAppend #my-path-to-choir #v-three {}}
     >>
     }
   #})

Duet =
#(define-music-function (parser location my-path-to-choir v-one v-two v-three)(string? string? string?)
   #{
     \shiftDurations #1 #0 {
     \new StaffGroup = ChoirStaff <<
       { \vocalStaff \stringAppend #my-path-to-choir #v-one {}}
       { \vocalStaff \stringAppend #my-path-to-choir #v-two {}}
     >>
     }
   #})



%%% Testing
%%%\putMusic "massInC/kyrie/superius/melody" \relative c'' { c2 d4 c }
%%%\putMusic "massInC/kyrie/superius/incipit" \relative c'' { c2 d4 c }
%%%\putMusic "massInC/kyrie/superius/lyrics" \lyricmode { Ky -- ri -- e }
%%%\putMusic "massInC/kyrie/altus/melody" \relative c'' { r4 g f e }
%%%\putMusic "massInC/kyrie/altus/incipit" \relative c'' { r4 g f e }
%%%\putMusic "massInC/kyrie/altus/lyrics" \lyricmode { Ky -- ri -- e }
%%%\putMusic "massInC/kyrie/tenor/melody" \relative c { \clef "G_8" e4 f( g) g }
%%%\putMusic "massInC/kyrie/tenor/incipit" \relative c { \clef "petrucci-c4" e4 f( g) g }
%%%\putMusic "massInC/kyrie/tenor/lyrics" \lyricmode { Ky -- ri -- e }
%%%\putMusic "massInC/kyrie/bassus/melody" \relative c { \clef "bass" c4 g4 c~ c }
%%%\putMusic "massInC/kyrie/bassus/incipit" \relative c { \clef "petrucci-f4"c4 g4 c~ c }
%%%\putMusic "massInC/kyrie/bassus/lyrics" \lyricmode { Ky -- ri -- e __ }
%%%
%%%\score {
%%%  \SATBGroup "massInC/kyrie"
%%%  \layout {\layoutOne}
%%%}